/*
 * loadmon - a load average monitor for Linux
 * Copyright (C) 1995-1998 Steven Pritchard <steve@silug.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>

void handler(int);
void checkload(int);

char load[80]="";
double highload=0;
int done=0;

void handler(int sig)
{
#ifdef DEBUG
    fprintf(stderr, "Caught signal %i.\n", sig);
#endif
    fprintf(stderr, "Highest load=%s", load);
    switch (sig)
    {
        case SIGINT:
            break;
        
        case SIGQUIT:
        case SIGTERM:
            done++;
    }
}

void main(void)
{
    FILE *loadavg;

    loadavg=fopen("/proc/loadavg", "r");
    if (!loadavg)
    {
        fprintf(stderr, "Unable to open /proc/loadavg.  Is /proc mounted?\n");
        exit(EXIT_FAILURE);
    }
    fclose(loadavg);

    signal(SIGTERM, handler);
    signal(SIGINT, handler);
    signal(SIGQUIT, handler);
    signal(SIGALRM, checkload);

    checkload(0);

    while (!done)
    {
        alarm(1);
        pause();
    }

    exit(EXIT_SUCCESS);
}

void checkload(int sig)
{
    FILE *loadavg;
    char line[80];
    char tmp[80];
    char *sloadnow;
    double loadnow;

    loadavg=fopen("/proc/loadavg", "r");
    if (loadavg)
    {
        fgets(line, 80, loadavg);
        fclose(loadavg);
        strcpy(tmp, line);
        sloadnow=strtok(tmp, " ");
        loadnow=atof(sloadnow);
        if ((sig==0) || (loadnow > highload))
        {
            highload=loadnow;
            strcpy(load, line);
        }
    }
}
